@@ -44,19 +44,29 @@ class Virtuoso(Store):
     transaction_aware = True
     def __init__(self, *av, **kw):
         super(Virtuoso, self).__init__(*av, **kw)
-        self.__prefix = {}
-        self.__namespace = {}
 
     def open(self, dsn):
         self.__dsn = dsn
         try:
             self._connection = pyodbc.connect(dsn)
             log.info("Virtuoso Store Connected: %s" % dsn)
+            self.__init_ns_decls__()
             return VALID_STORE
         except:
             log.error("Virtuoso Connection Failed:\n%s" % format_exc())
             return NO_STORE
 
+    def __init_ns_decls__(self):
+        self.__prefix = {}
+        self.__namespace = {}
+        cursor = self.cursor()
+        q = u"DB.DBA.XML_SELECT_ALL_NS_DECLS()"
+        for prefix, namespace in cursor.execute(q):
+            namespace = URIRef(namespace)
+            self.__prefix[namespace] = prefix
+            self.__namespace[prefix] = namespace
+        cursor.close()
+
     def cursor(self, isolation=READ_COMMITTED):
         cursor = self._connection.cursor()
         cursor.execute("SET TRANSACTION ISOLATION LEVEL %s" % isolation)
@@ -81,6 +91,7 @@ class Virtuoso(Store):
         except:
             log.error(u"Error Executing: %s" % q)
             raise
+        ## handle where e.args == ('S1TAT', 'Message...')
 
     def sparql_query(self, q, *av, **kw):
         def _construct(results):
@@ -155,7 +166,9 @@ class Virtuoso(Store):
             q += u'{ %(S)s %(P)s %(O)s } WHERE { %(S)s %(P)s %(O)s }' % query_bindings
         self.query(q)
 
-    def bind(self, prefix, namespace):
+    def bind(self, prefix, namespace, flags=1):
+        q = u"DB.DBA.XML_SET_NS_DECL ('%s', '%s', %s)" % (prefix, namespace, flags)
+        self.query(q)
         self.__prefix[namespace] = prefix
         self.__namespace[prefix] = namespace
 
